package com.xxfc.platform.vehicle.mapper;

import com.xxfc.platform.vehicle.entity.BranchCompanyStockInfo;
import com.xxfc.platform.vehicle.pojo.BranchCompanyStockInfoVo;
import com.xxfc.platform.vehicle.pojo.BranchCompanyStockSearchVo;
import org.apache.ibatis.annotations.Param;
import tk.mybatis.mapper.common.Mapper;

import java.util.List;

public interface BranchCompanyStockInfoMapper extends Mapper<BranchCompanyStockInfo> {
    List<BranchCompanyStockInfoVo> selectVoAll();

    List<BranchCompanyStockInfoVo> search(BranchCompanyStockSearchVo searchVo);

    int updateBalance(@Param("companyId") Integer companyId, @Param("balance") Integer balance, @Param("addr_city") Integer addr_city);

    List<Integer> selectAllPrice();
    int addCompamyList(@Param("list") List<BranchCompanyStockInfo> list);

    String selectInfoById(Integer id);
}